from zope.annotation import IAnnotations
from zope.interface import implements
from zope.interface import implementer
from plone.rfc822.interfaces import IPrimaryFieldInfo
from collective.flowplayer.browser.view import File as FileViewBase
from collective.flowplayer.events import VIDEO_EXTENSIONS
from collective.flowplayer.events import AUDIO_EXTENSIONS
from collective.flowplayer.metadata_extraction import parse_raw
from collective.flowplayer.metadata_extraction import scale_from_metadata
from collective.flowplayer.interfaces import IMediaInfo

MEDIA_INFO_KEY = "collective.flowplayer:mediainfo"

class MediaInfo(object):
    implements(IMediaInfo)

    def __init__(self, width=None, height=None, audio_only=True, **kw):
        self.width = width
        self.height = height
        self.audio_only = audio_only


@implementer(IMediaInfo)
def get_media_info(context):
    anno = IAnnotations(context)
    data = anno.get(MEDIA_INFO_KEY)
    if data is None:
        return None
    return MediaInfo(**data)

def update_media_info(context, event):
    info = IPrimaryFieldInfo(context)
    anno = IAnnotations(context)
    ext = None
    if info.value is not None:
        parts = info.value.filename.rsplit('.', 1)
        if len(parts) == 2:
            ext = '.' + parts[1]
    if ext in AUDIO_EXTENSIONS:
        anno[MEDIA_INFO_KEY] = dict(audio_only=True)
    elif ext in VIDEO_EXTENSIONS:
        f = info.value.open()
        metadata = parse_raw(f)
        height, width = scale_from_metadata(metadata)
        anno[MEDIA_INFO_KEY] = dict(audio_only=False, width=width, height=height)
        f.close()
    else:
        if MEDIA_INFO_KEY in anno:
            del anno[MEDIA_INFO_KEY]


class FlowplayerFileView(FileViewBase):
    def getFilename(self):
        info = IPrimaryFieldInfo(self.context)
        return info.value.filename



from Products.CMFCore.utils import getToolByName
from collective.flowplayer.interfaces import IFlowPlayerSite
from zope.interface.interfaces import IInterface
from zope.component import getSiteManager

def uninstall(self):
    portal = self
    setup_tool = getToolByName(portal, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile('profile-collective.flowplayer:uninstall')


    # Remove portal properties
    ptool = getToolByName(portal, 'portal_properties')
    if ptool.get('flowplayer_properties', None):
        del ptool['flowplayer_properties']

    kupu = getToolByName(portal, 'kupu_library_tool', None)
    if kupu is not None:
        paragraph_styles = list(kupu.getParagraphStyles())
    
        new_styles = [('autoFlowPlayer video', 'Video|div'),
                      ('autoFlowPlayer video image-left', 'Video (left)|div'),
                      ('autoFlowPlayer video image-right', 'Video (right)|div'),
                      ('autoFlowPlayer audio', 'Audio|div'),
                      ('autoFlowPlayer audio image-left', 'Audio (left)|div'),
                      ('autoFlowPlayer audio image-right', 'Audio (right)|div')]
        fp_styles = dict(new_styles)
        fixed_styles = []
        for style in paragraph_styles:
            css_class = style.split('|')[-1]
            if css_class not in fp_styles:
                fixed_styles.append(style)

        kupu.configure_kupu(parastyles=fixed_styles)
        
    # remove collective.flowplayer.interfaces.IFlowPlayerSite utility
    sm = getSiteManager(context=portal)
    if sm.queryUtility(IInterface, name=u'collective.flowplayer.interfaces.IFlowPlayerSite', default=False):
        sm.unregisterUtility(component=IFlowPlayerSite, provided=IInterface, name=u'collective.flowplayer.interfaces.IFlowPlayerSite')
    return "Ran all uninstall steps."



from zope.cachedescriptors import property
from zope.interface import alsoProvides, noLongerProvides
from zope.interface.interfaces import IInterface
from zope.component import getSiteManager

from collective.flowplayer.interfaces import IMediaInfo, IAudio, IVideo
from collective.flowplayer.metadata_extraction import parse_raw
from collective.flowplayer.metadata_extraction import scale_from_metadata

from Products.ATContentTypes import interface
from Products.Archetypes.interfaces import IObjectInitializedEvent

import urllib2
from StringIO import StringIO

from hachoir_core.stream import StreamError

VIDEO_EXTENSIONS = ['.f4b', '.f4p', '.f4v', '.flv', '.mp4', '.m4v', '.mov']
AUDIO_EXTENSIONS = ['.mp3']

def is_flowplayer_installed(object):
    sm = getSiteManager(context=object)
    return sm.queryUtility(IInterface, name=u'collective.flowplayer.interfaces.IFlowPlayerSite', default=False)

def remove_marker(object):
    changed = False
    if IAudio.providedBy(object):
        noLongerProvides(object, IAudio)
        changed = True
    if IVideo.providedBy(object):
        noLongerProvides(object, IVideo)
        changed = True
    if changed:
        object.reindexObject(idxs=['object_provides'])

class ChangeView(object):

    interface = None
    value = None
    file_handle = None

    def __init__(self, object, event):
        self.object = object
        # TODO: do we really need this different from object?
        self.content = content = event.object

        if not is_flowplayer_installed(content): return
        if not self.interface.providedBy(content): return
        if self.value is None:
            remove_marker(content)
            return

        ext = self.check_extension()
        if ext is None:
            remove_marker(content)
            return

        # set the view to flowplayer view
        # TODO: give up automatic view changing - it causes problems when
        #       product is deinstalled. instead provide a file_view for
        #       the interfaces IAudio and IVideo in combination with
        #       IFileContent and IATLink.
        if IObjectInitializedEvent.providedBy(event):
            content.setLayout('flowplayer')

        if ext in AUDIO_EXTENSIONS:
            self.handleAudio()
        elif ext in VIDEO_EXTENSIONS:
            self.handleVideo()

    def check_extension(self):
        for ext in AUDIO_EXTENSIONS + VIDEO_EXTENSIONS:
            if isinstance(self.filename, basestring) and self.filename.endswith(ext):
                return ext
        return None

    def handleAudio(self):
        if not IAudio.providedBy(self.content):
            alsoProvides(self.content, IAudio)
            self.object.reindexObject(idxs=['object_provides'])

    def handleVideo(self):
        handle = self.file_handle
        try:
            metadata = parse_raw(handle)
            height, width = scale_from_metadata(metadata)
            handle.close()
        except StreamError:
            height = width = None
        
        if not IVideo.providedBy(self.content):
            alsoProvides(self.content, IVideo)
            self.object.reindexObject(idxs=['object_provides'])

        if height and width:
            info = IMediaInfo(self.content)
            info.height = height
            info.width = width
            


class ChangeFileView(ChangeView):

    interface = interface.IFileContent

    @property.Lazy
    def value(self):
        return self.content.getField('file').getRaw(self.content)

    @property.Lazy
    def filename(self):
        filename = self.value.filename
        if isinstance(filename, basestring):
            filename = filename.lower()
        return filename

    @property.Lazy
    def file_handle(self):
        file_object = self.value
        try:
            # For blobs
            file_handle = file_object.getIterator()
        except AttributeError:
            file_handle = StringIO(str(file_object.data))
        return file_handle


class ChangeLinkView(ChangeView):

    interface = interface.IATLink

    @property.Lazy
    def value(self):
        return self.content.getRemoteUrl()

    @property.Lazy
    def filename(self):
        filename = self.value
        if isinstance(filename, basestring):
            filename = filename.lower()
        return filename

    @property.Lazy
    def file_handle(self):
        try:
            file_handle = StringIO(str(urllib2.urlopen(self.value).read()))
        except IOError:
            file_handle = StringIO()
        return file_handle




from zope import interface
from zope import component
from Acquisition import aq_inner
import urllib
import os

try:
    # python2.6
    import json
except ImportError:
    # python2.4
    import simplejson as json

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from collective.flowplayer.utils import properties_to_dict, \
                                        flash_properties_to_dict

from collective.flowplayer.interfaces import IFlowPlayable
from collective.flowplayer.interfaces import IMediaInfo, IFlowPlayerView

from plone.memoize.instance import memoize
from plone.memoize import view

class JavaScript(BrowserView):

    @view.memoize_contextless
    def portal_state(self):
        """ returns
            http://dev.plone.org/plone/browser/plone.app.layout/trunk/plone/app/layout/globals/portal.py
        """
        return component.getMultiAdapter((self.context, self.request), name=u"plone_portal_state")

    @property
    def flowplayer_properties(self):
        properties_tool = getToolByName(self.context, 'portal_properties')
        return getattr(properties_tool, 'flowplayer_properties', None)

    @property
    def flowplayer_properties_as_dict(self):
        portal_url = self.portal_state().portal_url()
        return properties_to_dict(self.flowplayer_properties,
                                  portal_url,
                                  ignore=['title',
                                          'loop',
                                          'initialVolumePercentage',
                                          'showPlaylist',])

    @property
    def flash_properties_as_dict(self):
        portal_url = self.portal_state().portal_url()
        return flash_properties_to_dict(self.flowplayer_properties,
                                        portal_url)

    def __call__(self, request=None, response=None):
        """ Returns global configuration of the Flowplayer taken from portal_properties """
        self.request.response.setHeader("Content-type", "text/javascript")
        data = dict(
            config=self.flowplayer_properties_as_dict,
            params=self.flash_properties_as_dict,
            loop=bool(self.flowplayer_properties.getProperty('loop')),
            )
        volume = self.flowplayer_properties.getProperty('initialVolumePercentage')
        if volume:
            data['initialVolumePercentage'] = int(volume)
        else:
            data['initialVolumePercentage'] = None
        return "var collective_flowplayer = %s;" % json.dumps(data, indent=2)


class File(BrowserView):
    interface.implements(IFlowPlayerView)

    def __init__(self, context, request):
        super(File, self).__init__(context, request)

        self.info = IMediaInfo(self.context, None)

        self.height = self.info is not None and self.info.height or None
        self.width = self.info is not None and self.info.width or None
        self._audio_only = self.info is not None and self.info.audio_only or None

        if self.height and self.width:
            self._scale = "height: %dpx; width: %dpx;" % (self.height, self.width)
        else:
            self._scale = ""

    def audio_only(self):
        return self._audio_only

    def scale(self):
        return self._scale

    def videos(self):
        return[dict(url=self.href(),
                    title=self.context.Title(),
                    description=self.context.Description(),
                    height=self.height,
                    width=self.width,
                    audio_only=self._audio_only)]

    def getFilename(self):
        context = aq_inner(self.context)
        return context.getFilename()

    def href(self):
        context = aq_inner(self.context)
        ext = ''
        url = self.context.absolute_url()
        filename = self.getFilename()
        if filename:
            extension = os.path.splitext(filename)[1]
            if not url.endswith(extension):
                ext = "?e=%s" % extension
        return self.context.absolute_url()+ext

class Link(File):

    def href(self):
        return self.context.getRemoteUrl()

class Folder(BrowserView):
    interface.implements(IFlowPlayerView)

    @memoize
    def playlist_class(self):
        properties_tool = getToolByName(self.context, 'portal_properties')
        props = getattr(properties_tool, 'flowplayer_properties', None)
        return props and props.getProperty('showPlaylist') and 'flowPlaylistVisible' or 'flowPlaylistHidden'

    @memoize
    def audio_only(self):
        return len([v for v in self.videos() if not v['audio_only']]) == 0

    @memoize
    def scale(self):
        height = 0
        width = 0
        if self.audio_only():
            height = 27
            width = 400

        for video in self.videos():
            if video['height'] > height or video['width'] > width:
                height = video['height']
                width = video['width']

        if height and width:
            return "height: %dpx; width: %dpx;" % (height, width)

    @memoize
    def videos(self):
        results = []
        for brain in self._query():
            video = brain.getObject()
            if not IFlowPlayable.providedBy(video):
                continue
            view = component.getMultiAdapter(
                (video, self.request), interface.Interface, 'flowplayer')
            results.append(dict(url=view.href(),
                                title=brain.Title,
                                description=brain.Description,
                                height=view.height,
                                width=view.width,
                                audio_only=view.audio_only()))
        return results

    def first_clip_url(self):
        """ Clip must be quoted to playlist is able to find it in the flowplayer-playlist onBegin/getEl method call """
        videos = self.videos()
        if videos:
            return urllib.quote(videos[0].get('url'))
        else:
            return None

    def _query(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(object_provides=IFlowPlayable.__identifier__,
                       path = {'depth': 1, 'query': '/'.join(self.context.getPhysicalPath())},
                       sort_on='getObjPositionInParent')

class Topic(Folder):
    interface.implements(IFlowPlayerView)

    def _query(self):
        return self.context.queryCatalog()



import unittest
import time
import os

testfile_home = os.path.join(os.path.dirname(__file__))
file_handle = open(testfile_home + '/barsandtone.flv', 'rb')

class TestMetadataProfiling(unittest.TestCase):

    def testFLVHeader(self):
        tt = time.time()
        # import
        from collective.flowplayer.flv import FLVHeader
        # parse
        file_handle.seek(0)
        flvparser = FLVHeader()
        flvparser.analyse(file_handle.read(1024))
        assert((288, 360) == (flvparser.getHeight(), flvparser.getWidth()))
        print 'parse with FLVHeader: ' + str(time.time() - tt)

    def testHachoir(self):
        tt = time.time()
        # import
        import collective.flowplayer.metadata_extraction as metaex
        # parse
        file_handle.seek(0)
        metadata = metaex.parse_raw(file_handle)
        file_handle.close()
        assert((288, 360) == metaex.scale_from_metadata(metadata))
        print 'parse with metadata_extraction: ' + str(time.time() - tt)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMetadataProfiling))
    return suite



from setuptools import setup, find_packages
import os
import sys

install_requires = [
          'setuptools',
          'plone.app.jquerytools',
          # 'Plone', Too confusing for those using releases < 3.2
          'hachoir_core',
          'hachoir_metadata',
          'hachoir_parser',
      ]

version = '3.0'

tests_require = ['collective.testcaselayer', 'interlude']

if sys.version_info < (2, 6):
    install_requires.append('simplejson')

setup(name='collective.flowplayer',
      version=version,
      description="A simple package using Flowplayer for video and audio content",
      long_description=open("README.txt").read() + "\n\n" +
                       open(os.path.join("docs", "UPGRADE.txt")).read() + "\n\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='flv flash video plone',
      author='Martin Aspeli',
      author_email='optilude@gmail.com',
      url='http://plone.org/products/collective-flowplayer',
      license='GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir = {'':'src'},
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      tests_require=tests_require,
      extras_require={'test': tests_require},
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



import logging
logger = logging.getLogger('collective.flowplayer.metadata_extraction')

from hachoir_parser.guess import createParser, guessParser
from hachoir_metadata.metadata import extractMetadata
from hachoir_core.error import HachoirError
from hachoir_core.stream import InputStreamError, InputIOStream

def make_unicode(string):
    if not isinstance(string, unicode):
        string = string.decode('utf-8')
    return string

def parse_raw(raw):
    stream = InputIOStream(raw)
    parser = guessParser(stream)
    return extract_metadata(parser)

def parse_file(filename):
    filename = make_unicode(filename)
    try:
        parser = createParser(filename)
    except InputStreamError, err:
        logger.error("stream error! %s\n" % unicode(err))
        return None
    return extract_metadata(parser)

def extract_metadata(parser):
    if not parser:
        logger.error("Unable to create parser.\n")
        return None
    try:
        metadata = extractMetadata(parser)
    except HachoirError, err:
        logger.error("stream error! %s\n" % unicode(err))
        return None

    if metadata is None:
        logger.error("unable to extract metadata.\n")
        return None

    return metadata

def scale_from_metadata(metadata):
    try:
        height = metadata.get('height', None)
        width = metadata.get('width', None)
    except (ValueError, AttributeError):
        return None, None
    return height, width



import unittest
import doctest

from Testing import ZopeTestCase
from Products.PloneTestCase import ptc
from collective.testcaselayer import ptc as tcl_ptc
from Products.Five import zcml

from interlude import interact

optionflags = (doctest.NORMALIZE_WHITESPACE|
               doctest.ELLIPSIS|
               doctest.REPORT_NDIFF)

ptc.setupPloneSite()
class Layer(tcl_ptc.BasePTCLayer):
    """Install collective.flowplayer"""
    def afterSetUp(self):
        import collective.flowplayer
        zcml.load_config('configure.zcml', collective.flowplayer)
        self.addProfile('collective.flowplayer:default')
        # put resource registry to debug mode to avoid cachekyes in tests
        self.portal.portal_css.setDebugMode(True)
        self.portal.portal_javascripts.setDebugMode(True)
layer = Layer([tcl_ptc.ptc_layer])


FUNCTIONALTESTFILES = [
    'README.txt',
]
TESTFILES = [
    '../metadata_extraction.txt',
]

def test_suite():
    test_class = ptc.FunctionalTestCase
    test_class.layer = layer

    return unittest.TestSuite(
        [ZopeTestCase.FunctionalDocFileSuite(
            file,
            package='collective.flowplayer',
            optionflags=optionflags,
            globs={'interact': interact},
            test_class=test_class
        ) for file in FUNCTIONALTESTFILES]

        + [doctest.DocFileSuite(
            file,
            optionflags=optionflags,
            globs={'interact': interact},
        ) for file in TESTFILES]
    )

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')


import unittest
import os.path

import zope.interface

from Products.PloneTestCase import ptc

from collective.flowplayer.tests import tests
from collective.flowplayer.interfaces import IMediaInfo, IVideo
from collective.flowplayer.events import ChangeLinkView

from Products.CMFPlone.utils import _createObjectByType

class StubEvent(object):

    def __init__(self, object):
        self.object = object

class ScalableLinkTest(ptc.PloneTestCase):
    """ Test fetching of metadata of remote videos """

    layer = tests.Layer

    def test_link_dimensions(self):
        _createObjectByType('Link', self.folder, 'my-video-link')
        link = self.folder['my-video-link']
        link.setRemoteUrl('file://%s' % os.path.join(os.path.dirname(__file__),
                                                    'barsandtone.flv'))
        zope.interface.alsoProvides(link, IVideo)

        self.failUnless(IMediaInfo(link).width is None)
        self.failUnless(IMediaInfo(link).height is None)

        ChangeLinkView(link, StubEvent(link))

        self.failUnlessEqual(IMediaInfo(link).width, 360)
        self.failUnlessEqual(IMediaInfo(link).height, 288)


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
